<?php


use VerifoneEcomAPI\ApiWrapper\Settings;

/**
 * Class Verifone_Checkout
 */
class Verifone_Checkout extends Verifone_Api_Entity
{
    const SUPPORTED_I18N_LANGUAGES = ['en', 'da', 'fi', 'fr', 'he', 'no', 'pl', 'pt', 'sv'];
    public $cart;
    private $customer_id;
    private $helper;
    private $settings;
    private $params;
    protected $data = [];
    protected $precision;
    protected $transactionType;

    /**
     * Verifone_Checkout constructor.
     *
     * @param $params
     * @param  Settings  $settings
     * @param  Verifone_Helper  $helper
     * @param  string  $customer_id
     *
     * @throws Exception
     */
    public function __construct(
        $params,
        Settings $settings,
        Verifone_Helper $helper,
        string $customer_id = ''
    ) {

        $this->params          = $params;
        $this->cart            = $params['cart'];
        $this->customer_id     = $customer_id;
        $this->helper          = $helper;
        $this->settings        = $settings;
        $this->precision       = $this->getCurrencyPrecision($params['currency']->iso_code);
        $this->transactionType = $this->helper->getDeviceType();
        $this->buildData();
    }

    /**
     * @throws Exception
     */
    private function buildData()
    {

        $this->data = [
            'customer'           => $this->customer_id,
            'amount'             => (int) $this->getFormattedAmount(),
            'currency_code'      => strtoupper($this->params['currency']->iso_code),
            'entity_id'          => $this->settings->getEntityId(),
            'merchant_reference' => (string) $this->cart->id,
            'return_url'         => $this->params['returnUrl'],
            'i18n'               => $this->getLanguage(),
            'interaction_type'   => strtoupper($this->settings->getCheckoutType()),
            'configurations'     => []
        ];
        $this->card();
        $this->paypal();
        $this->theme();
        $this->applePay();
        $this->googlePay();
    }

    /**
     * This function NEEDS to return a string then to be converted into 'int' because php transforms - 19.99 to 1998
     * Formats the amount value
     *
     * @return string
     */
    public function getFormattedAmount()
    : string
    {

        return (string) (Tools::ps_round((float) $this->cart->getOrderTotal(true, CartCore::BOTH), 2) * 100);
    }

    /**
     * @return array
     */
    public function toArray()
    : array
    {
        return $this->data;
    }

    /**
     * @return false|string
     */
    public function toJson()
    {
        return json_encode($this->data);
    }

    public function getCurrencyPrecision($currencyCode)
    : int {
        $payment_currency_id = Currency::getIdByIsoCode($currencyCode);
        $payment_currency    = Currency::getCurrencyInstance($payment_currency_id);

        return $payment_currency->precision;
    }

    /**
     * @return array
     */
    private function getLanguage()
    : array
    {
        $lang = Language::getLanguage((int) Configuration::get('PS_LANG_DEFAULT'))['iso_code'];

        return [
            'default_language'      => in_array(self::SUPPORTED_I18N_LANGUAGES, $lang) ? $lang : 'en',
            'fallback_language'     => 'en',
            'show_language_options' => true,
        ];
    }

    /**
     * Card payments
     * @return void
     */
    private function card()
    {
        if ($this->settings->getCardEnabled()) {
            $this->data['configurations']['card'] = [
                'capture_now'         => true,
                'payment_contract_id' => $this->settings->getPaymentContractId(),
            ];

            if ($this->settings->getThreeDSEnabled()) {
                $this->data ['configurations']['card']['threed_secure'] = [
                    'threeds_contract_id' => $this->settings->getThreeDSContractId(),
                    'enabled'             => true,
                    'transaction_mode'    => $this->transactionType,
                ];
            }
        }
    }

    /**
     * PayPal payments
     * @return void
     */
    private function paypal()
    {
        if ($this->settings->getPaypalEnabled()) {
            $this->data['configurations']['paypal'] = [
                'capture_now'         => true,
                'payment_contract_id' => $this->settings->getPaypalContractId(),
            ];
        }
    }

    /**
     * ThemeId - if not empty then we send it
     * @return void
     */
    private function theme()
    {
        if ( ! empty($this->settings->getThemeId())) {
            $this->data['theme_id'] = $this->settings->getThemeId();
        }
    }

    /**
     * ApplePay - APM
     * @return void
     */
    private function applePay()
    {
        if ($this->settings->getApplePayEnabled()) {
            $this->data['configurations']['apple_pay'] = [
                'capture_now' => true,
                'card'        => [
                    'sca_compliance_level' => $this->settings->getApplePaySCALevel(),
                    'payment_contract_id'  => $this->settings->getApplePayContractId(),
                ]
            ];

            if ($this->settings->getApplePaySCALevel() !== 'NONE') {
                $this->data['configurations']['apple_pay']['card']['threed_secure'] = [
                    'threeds_contract_id' => $this->settings->getApplePayThreeDSContractId(),
                    'transaction_mode'    => $this->transactionType,
                ];
            }
        }
    }

    /**
     * GoolgePay - APM
     * @return void
     */
    private function googlePay()
    {
        if ($this->settings->getGooglePayEnabled()) {
            $this->data['configurations']['google_pay'] = [
                'capture_now' => true,
                'card'        => [
                    'sca_compliance_level' => $this->settings->getGooglePaySCALevel(),
                    'payment_contract_id'  => $this->settings->getGooglePayContractId(),
                ]
            ];

            if ($this->settings->getGooglePaySCALevel() !== 'NONE') {
                $this->data['configurations']['google_pay']['card']['threed_secure'] = [
                    'threeds_contract_id' => $this->settings->getGooglePayThreeDSContractId(),
                    'transaction_mode'    => $this->transactionType,
                ];
            }
        }
    }
}
